#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-19
#
# monitor Check Point S2S VPNs
#
# 2016-07-24: added missing tunnel handling
# 2016-07-28: fix for missing item
# 2018-05-02: code cleanup, rewrite of check and inventory function
# 2020-06-08: changed snmp-scan function
# 2021-09-01: rewritten for CMK 2.0
# 2022-10-23: fixed warning in upgrade "non-empty params vanished" for vpn_entry
# 2023-05-29: moved gui files to ~/local/lib/check_mk/gui/plugins/...
# 2023-10-28: fixed crash on empty l2lpeerobjname (THX to andreas doehler)
#             renamed parameters. needs tabularasa
#             removed ":" from service name
#             some cleanup

from dataclasses import dataclass
from typing import List, Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    OIDEnd,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointL2lPeer:
    peer_ipaddr: str
    peer_obj_name: str
    state: int
    community: str
    next_hop: str
    interface: str
    source_ip_addr: str
    link_priority: int
    peer_type: int
    type: int
    probe_state: int


_permanentTunnelState = {
    3: 'active',
    4: 'destroy',
    129: 'idle',
    130: 'phase1',
    131: 'down',
    132: 'init',
}

_permanentTunnelLinkPriority = {
    0: 'primary',
    1: 'backup',
    2: 'on-demand',
}

_permanentTunnelProbState = {
    0: 'unknown',
    1: 'alive',
    2: 'dead',
}

_permanentTunnelPeerType = {
    1: 'regular',
    2: 'daip',
    3: 'robo',
}

_tunnelType = {
    1: 'regular',
    2: 'permanent',
}


def parse_checkpoint_l2l_vpn(string_table: List[StringTable]) -> Dict[str, CheckpointL2lPeer]:
    parsed: Dict[str, CheckpointL2lPeer] = {}

    def add_vpn_entry():
        parsed[peer_obj_name] = CheckpointL2lPeer(
            peer_ipaddr=peer_ip_addr,
            peer_obj_name=peer_obj_name,
            state=int(state),
            community=community,
            next_hop=next_hop,
            interface=interface,
            source_ip_addr=source_ip_addr,
            link_priority=int(link_priority),
            peer_type=int(peer_type),
            type=int(l2l_type),
            probe_state=int(probe_state),
        )

    # non permanent tunnel
    for vpn_entry in string_table[0]:
        peer_ip_addr, peer_obj_name, state, community, next_hop, interface, source_ip_addr, \
            link_priority, probe_state, peer_type, l2l_type = vpn_entry
        peer_ip_addr = peer_ip_addr[:-2]
        if peer_obj_name:
            add_vpn_entry()

    # permanent tunnel
    for vpn_entry in string_table[1]:
        peer_ip_addr, peer_obj_name, state, community, next_hop, interface, source_ip_addr, \
            link_priority, probe_state, peer_type = vpn_entry
        peer_ip_addr = peer_ip_addr[:-2]
        l2l_type = u'2'
        if peer_obj_name:
            add_vpn_entry()

    return parsed


def discovery_checkpoint_l2l_vpn(section: Dict[str, CheckpointL2lPeer]) -> DiscoveryResult:
    for l2lpeerobjname in section.keys():
        yield Service(
            item=l2lpeerobjname,
            parameters={'vpn_entry': {
                'community': section[l2lpeerobjname].community,
                'peer_ip_addr': section[l2lpeerobjname].peer_ipaddr,
            }}
        )


def check_checkpoint_l2l_vpn(item, params, section: Dict[str, CheckpointL2lPeer]) -> CheckResult:
    not_found_state = params['not_found_state']

    for peerobject, tunnel_alias, tunnel_not_found_state in params.get('tunnels', []):
        if item == peerobject:
            yield Result(state=State.OK, summary=f'[{tunnel_alias}]')
            not_found_state = tunnel_not_found_state

    try:
        vpn_entry = section[item]
    except KeyError:
        vpn_entry = params['vpn_entry']
        yield Result(
            state=State(not_found_state),
            notice=f'Tunnel not found. Expected: '
                   f'Community: {vpn_entry["community"]}, '
                   f'Peer IP: {vpn_entry["peer_ip_addr"]}'
        )
        return

    yield Result(state=State.OK, summary=f'Community: {vpn_entry.community}')
    yield Result(state=State.OK, summary=f'Peer IP: {vpn_entry.peer_ipaddr}')

    if vpn_entry.source_ip_addr != '0.0.0.0':
        yield Result(state=State.OK, summary=f'Local IP: {vpn_entry.source_ip_addr}')

    if vpn_entry.next_hop != '0.0.0.0':
        yield Result(state=State.OK, summary=f'Next hop IP: {vpn_entry.next_hop}')

    if vpn_entry.interface != '':
        yield Result(state=State.OK, summary=f'Local interface: {vpn_entry.interface}')

    # backup or on-demand tunnel
    if vpn_entry.link_priority == 1:
        yield Result(
            state=State.warn,
            summary=f'Tunnel priority: {_permanentTunnelLinkPriority.get(vpn_entry.link_priority)}'
        )
    elif vpn_entry.link_priority != 0:
        yield Result(
            state=State.OK,
            summary=f'Tunnel priority: {_permanentTunnelLinkPriority.get(vpn_entry.link_priority, "unknown")}'
        )

    # not non permanent tunnel
    if vpn_entry.type != 1:
        yield Result(state=State.OK, summary=f'Tunnel type: {_tunnelType.get(vpn_entry.type)}')

    # daip or robo tunnel
    if vpn_entry.peer_type != 1:
        yield Result(state=State.OK, summary=f'Peer type: {_permanentTunnelPeerType.get(vpn_entry.peer_type)}')

    # tunnel probe state 'dead'
    if vpn_entry.probe_state == 2:
        yield Result(state=State.CRIT, summary='Tunnel probe state "dead"')

    # tunnel state 'down'
    if vpn_entry.state == 131:
        yield Result(state=State.CRIT, summary='Tunnel down')

    # tunnel state not active
    elif vpn_entry.state != 3:
        yield Result(
            state=State.WARN,
            summary=f'Tunnel state: {_permanentTunnelState.get(vpn_entry.state, "unknown")}'
        )

    yield Result(
        state=State.OK,
        notice=f'Tunnel state: {_permanentTunnelState.get(vpn_entry.state, vpn_entry.state)}'
    )
    yield Result(
        state=State.OK,
        notice=f'Tunnel probe state: {_permanentTunnelProbState.get(vpn_entry.probe_state, vpn_entry.probe_state)}'
    )
    yield Result(state=State.OK, notice=f'Tunnel peer IP: {vpn_entry.peer_ipaddr}')
    yield Result(state=State.OK, notice=f'Tunnel local IP: {vpn_entry.source_ip_addr}')
    yield Result(state=State.OK, notice=f'Tunnel next hop IP: {vpn_entry.next_hop}')
    yield Result(state=State.OK, notice=f'Tunnel local interface: {vpn_entry.interface}:')
    yield Result(
        state=State.OK,
        notice=f'Tunnel priority: {_permanentTunnelLinkPriority.get(vpn_entry.link_priority)}'
    )
    yield Result(state=State.OK, notice=f'Tunnel type: {_tunnelType.get(vpn_entry.type)}')
    yield Result(state=State.OK, notice=f'Peer type: {_permanentTunnelPeerType.get(vpn_entry.peer_type)}')
    yield Result(state=State.OK, notice=f'Community: {vpn_entry.community}')


register.snmp_section(
    name='checkpoint_l2l_vpn',
    parse_function=parse_checkpoint_l2l_vpn,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.500.9002.1',  # CHECKPOINT-MIB
            oids=[
                OIDEnd(),  # Tunnel Peer IP
                '2',  # tunnelPeerObjName
                '3',  # tunnelState
                '4',  # tunnelCommunity
                '5',  # tunnelNextHop
                '6',  # tunnelInterface
                '7',  # tunnelSourceIpAddr
                '8',  # tunnelLinkPriority
                '9',  # tunnelProbState
                '10',  # tunnelPeerType
                '11',  # tunnelType
            ]),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.500.9003.1',  # CHECKPOINT-MIB
            oids=[
                OIDEnd(),  # Tunnel Peer IP
                '2',  # permanentTunnelPeerObjName
                '3',  # permanentTunnelState
                '4',  # permanentTunnelCommunity
                '5',  # permanentTunnelNextHop
                '6',  # permanentTunnelInterface
                '7',  # permanentTunnelSourceIpAddr
                '8',  # permanentTunnelLinkPriority
                '9',  # permanentTunnelProbState
                '10',  # permanentTunnelPeerType
            ]),
    ],
    detect=all_of(
        any_of(
            startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
            all_of(
                equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
                equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
            )
        ),
        equals('.1.3.6.1.4.1.2620.1.1.21.0', 'Firewall'),
    )
)

register.check_plugin(
    name='checkpoint_l2l_vpn',
    service_name='L2L VPN %s',
    discovery_function=discovery_checkpoint_l2l_vpn,
    check_function=check_checkpoint_l2l_vpn,
    check_ruleset_name='checkpoint_l2l_vpn',
    check_default_parameters={
        'not_found_state': 3,    # default state for tunnel not found
        'tunnels': [],  # list of tunnel specific not found states ('<ip-address>', '<alias>', <state>)
        # 'vpn_entry': {},
    },
)
